/******************************************************************************//**
 * @file     hid_transfer.c
 * @version  V3.00
 * @brief    NUC121 series USBD HID transfer sample file
 *
 * @copyright (C) 2016 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/

/*!<Includes */
#include <stdio.h>
#include <string.h>
#include "NUC121.h"
#include "hid_transfer.h"

volatile uint8_t set_report_flag = 0;
volatile uint8_t get_set_report_flag = 0;

extern uint8_t MainVolume, LeftChannelVolume, RightChannelVolume;

void USBD_IRQHandler(void)
{
    uint32_t u32IntSts = USBD_GET_INT_FLAG();
    uint32_t u32State = USBD_GET_BUS_STATE();
	
    //------------------------------------------------------------------
    if (u32IntSts & USBD_INTSTS_FLDET) {
        // Floating detect
        USBD_CLR_INT_FLAG(USBD_INTSTS_FLDET);

        if (USBD_IS_ATTACHED()) {
            /* USB Plug In */
            USBD_ENABLE_USB();

            /*Enable HIRC tirm*/
            SYS->IRCTCTL = DEFAULT_HIRC_TRIM_SETTING;
        } else {
            /* USB Un-plug */
            USBD_DISABLE_USB();

            /*Disable HIRC tirm*/
            SYS->IRCTCTL = DEFAULT_HIRC_TRIM_SETTING & (~SYS_IRCTCTL_FREQSEL_Msk);
        }
    }

    //------------------------------------------------------------------
    if (u32IntSts & USBD_INTSTS_BUS) {
        /* Clear event flag */
        USBD_CLR_INT_FLAG(USBD_INTSTS_BUS);

        if (u32State & USBD_STATE_USBRST) {
            /* Bus reset */
            USBD_ENABLE_USB();
            USBD_SwReset();

            /*Enable HIRC tirm*/
            SYS->IRCTCTL = DEFAULT_HIRC_TRIM_SETTING;
        }

        if (u32State & USBD_STATE_SUSPEND) {
            /* Enable USB but disable PHY */
            USBD_DISABLE_PHY();

            /*Disable HIRC tirm*/
            SYS->IRCTCTL = DEFAULT_HIRC_TRIM_SETTING & (~SYS_IRCTCTL_FREQSEL_Msk);

        }

        if (u32State & USBD_STATE_RESUME) {
            /* Enable USB and enable PHY */
            USBD_ENABLE_USB();

            /*Enable HIRC tirm*/
            SYS->IRCTCTL = DEFAULT_HIRC_TRIM_SETTING;
        }

#ifdef SUPPORT_LPM

        if (u32State & USBD_ATTR_L1SUSPEND_Msk) {
            /*
               TODO: Implement LPM SUSPEND flag here.
                     Remember to implement the power-saving function in main loop.
            */
        }

        if (u32State & USBD_ATTR_L1RESUME_Msk) {
            /*
               TODO: Implement LPM RESUME flag here.
            */
        }

#endif


    }

    if (u32IntSts & USBD_INTSTS_NEVWKIF_Msk) {
        /*Clear no-event wake up interrupt */
        USBD_CLR_INT_FLAG(USBD_INTSTS_NEVWKIF_Msk);
        /*
           TODO: Implement the function that will be executed when device is woken by non-USB event.
        */
    }

    //------------------------------------------------------------------
    if (u32IntSts & USBD_INTSTS_USB) {
        // USB event
        if (u32IntSts & USBD_INTSTS_SETUP) {
            // Setup packet
            /* Clear event flag */
            USBD_CLR_INT_FLAG(USBD_INTSTS_SETUP);

            /* Clear the data IN/OUT ready flag of control end-points */
            USBD_STOP_TRANSACTION(EP0);
            USBD_STOP_TRANSACTION(EP1);

            USBD_ProcessSetupPacket();
        }

        // EP events
        if (u32IntSts & USBD_INTSTS_EP0) {
            /* Clear event flag */
            USBD_CLR_INT_FLAG(USBD_INTSTS_EP0);
            // control IN
            USBD_CtrlIn();
        }

        if (u32IntSts & USBD_INTSTS_EP1) {
            /* Clear event flag */
            USBD_CLR_INT_FLAG(USBD_INTSTS_EP1);

            // control OUT
            USBD_CtrlOut();
						
						/* Nuvoton */
						if(set_report_flag == 1)
								get_set_report_flag = 1;
        }

				if(u32IntSts & USBD_INTSTS_EP2)
        {
            /* Clear event flag */
            USBD_CLR_INT_FLAG(USBD_INTSTS_EP2);
            // Interrupt IN
            EP2_Handler();
        }

        if(u32IntSts & USBD_INTSTS_EP3)
        {
            /* Clear event flag */
            USBD_CLR_INT_FLAG(USBD_INTSTS_EP3);
            // Interrupt OUT
            EP3_Handler();
        }

        if (u32IntSts & USBD_INTSTS_EP4) {
            /* Clear event flag */
            USBD_CLR_INT_FLAG(USBD_INTSTS_EP4);
        }

        if (u32IntSts & USBD_INTSTS_EP5) {
            /* Clear event flag */
            USBD_CLR_INT_FLAG(USBD_INTSTS_EP5);
        }

        if (u32IntSts & USBD_INTSTS_EP6) {
            /* Clear event flag */
            USBD_CLR_INT_FLAG(USBD_INTSTS_EP6);
        }

        if (u32IntSts & USBD_INTSTS_EP7) {
            /* Clear event flag */
            USBD_CLR_INT_FLAG(USBD_INTSTS_EP7);
        }
    }
}

void EP2_Handler(void)  /* Interrupt IN handler */
{

}

void EP3_Handler(void)  /* Interrupt OUT handler */
{
    uint8_t *ptr;
    /* Interrupt OUT */
    ptr = (uint8_t *)(USBD_BUF_BASE + USBD_GET_EP_BUF_ADDR(EP3));
    HID_GetOutReport(ptr, USBD_GET_PAYLOAD_LEN(EP3));
    USBD_SET_PAYLOAD_LEN(EP3, EP3_MAX_PKT_SIZE);
}

/*--------------------------------------------------------------------------*/
/**
  * @brief  USBD Endpoint Config.
  * @param  None.
  * @retval None.
  */
void HID_Init(void)
{
    /* Init setup packet buffer */
    /* Buffer range for setup packet -> [0 ~ 0x7] */
    USBD->STBUFSEG = SETUP_BUF_BASE;

    /*****************************************************/
    /* EP0 ==> control IN endpoint, address 0 */
    USBD_CONFIG_EP(EP0, USBD_CFG_CSTALL | USBD_CFG_EPMODE_IN | 0);
    /* Buffer range for EP0 */
    USBD_SET_EP_BUF_ADDR(EP0, EP0_BUF_BASE);

    /* EP1 ==> control OUT endpoint, address 0 */
    USBD_CONFIG_EP(EP1, USBD_CFG_CSTALL | USBD_CFG_EPMODE_OUT | 0);
    /* Buffer range for EP1 */
    USBD_SET_EP_BUF_ADDR(EP1, EP1_BUF_BASE);

    /*****************************************************/
    /* EP2 ==> Interrupt IN endpoint, address 1 */
    USBD_CONFIG_EP(EP2, USBD_CFG_EPMODE_IN | INT_IN_EP_NUM);
    /* Buffer range for EP2 */
    USBD_SET_EP_BUF_ADDR(EP2, EP2_BUF_BASE);

		/* EP3 ==> Interrupt OUT endpoint, address 2 */
    USBD_CONFIG_EP(EP3, USBD_CFG_EPMODE_OUT | INT_OUT_EP_NUM);
    /* Buffer range for EP3 */
    USBD_SET_EP_BUF_ADDR(EP3, EP3_BUF_BASE);
    /* trigger to receive OUT data */
    USBD_SET_PAYLOAD_LEN(EP3, EP3_MAX_PKT_SIZE);
}

uint8_t SendBuffer[128], g_SendLen;
int32_t HID_CtrlSetupGetReport(void)
{
		uint8_t buf[8];
		uint16_t	len;
		int i;
		USBD_MemCopy(buf, (uint8_t *)USBD_BUF_BASE, 8);

		for(i = 0; i < 128; i++)
				SendBuffer[i] = i;
		g_SendLen = 128;
    
		len = (buf[7] << 8) | buf[6];
		if(len >= g_SendLen)
        len = g_SendLen;

		USBD_PrepareCtrlIn((uint8_t *)&SendBuffer[0], len);
		USBD_PrepareCtrlOut(0,0);
	
    return TRUE;
}

uint8_t RcvBuffer[256];
int32_t HID_CtrlSetupSetReport(void)
{
		uint8_t buf[8];
    uint8_t report_type;
		uint32_t u32Len;

		USBD_MemCopy(buf, (uint8_t *)USBD_BUF_BASE, 8);
    report_type= buf[3];
		u32Len = (buf[7] << 8) | buf[6];

		if (report_type == 1)
		{
				/* EN: Report Type = input */
				// Trigger next Control Out DATA1 Transaction.
				USBD_SET_DATA1(EP1);
				USBD_SET_PAYLOAD_LEN(EP1, 0);

				//DBG_PRINTF(" - Input\n");
		}
		else if (report_type == 2)
		{
				/* EN: Report Type = Output */
				USBD_PrepareCtrlOut((uint8_t *)&RcvBuffer, u32Len);
				USBD_PrepareCtrlIn(0,0);

				//DBG_PRINTF(" - Output\n");
		}
		else if (report_type == 3)
		{
				/* Request Type = Feature */
//				USBD_SET_DATA1(EP1);
//				USBD_SET_PAYLOAD_LEN(EP1, 0);
				USBD_PrepareCtrlOut((uint8_t *)&RcvBuffer, u32Len);
				USBD_PrepareCtrlIn(0,0);
			
				set_report_flag = 1;
				//DBG_PRINTF(" - Feature\n");
		}
		else
		{
				// Not support. Reply STALL.
				USBD_SetStall(0);
				USBD_SetStall(1);
			
				//DBG_PRINTF(" - Unknown\n");
		}

		return TRUE;
}

void HID_ClassRequest(void)
{
    uint8_t buf[8];

    USBD_GetSetupPacket(buf);

    if (buf[0] & 0x80) { /* request data transfer direction */
        // Device to host
        switch (buf[1])
				{
						case GET_REPORT:
//						{
//								HID_CtrlSetupGetReport();
//								break;
//						}
						case GET_IDLE:
//		        {
//								break;
//		        }
						case GET_PROTOCOL:
//		        {
//								break;
//		        }
						default:
						{
								/* Setup error, stall the device */
								USBD_SetStall(0);
								break;
						}
        }
    } else {
        // Host to device
        switch (buf[1])
				{
						case SET_REPORT:
						{
								HID_CtrlSetupSetReport();
								break;
						}

						case SET_IDLE:
						{
								/* Status stage */
								USBD_SET_DATA1(EP0);
								USBD_SET_PAYLOAD_LEN(EP0, 0);
								break;
						}
						case SET_PROTOCOL:
//						{
//								break;
//						}
						default:
						{
								// Stall
								/* Setup error, stall the device */
								USBD_SetStall(0);
								break;
						}
        }
    }
}

/***************************************************************/
/* HID Transfer Commands */
#define HID_CMD_ACK					0x5A
#define HID_CMD_ISP					0xA0
#define HID_CMD_GETFW				0xB0
#define HID_CMD_GETPARTNO		0xB2
#define HID_CMD_AudioVolume	0xC0

typedef __packed struct
{
		/* Nuvoton_Henry */
		uint8_t ReportID;
		/* Nuvoton_Henry */
    uint8_t u8Cmd;
	  uint8_t Version;
		uint8_t u8AudioMain;
		uint8_t u8AudioLeft;
		uint8_t u8AudioRight;
		/* Reserved data for future use */
		uint8_t Data[58];
} CMD_T;

CMD_T gCmd;

void HID_GetOutReport(uint8_t *pu8EpBuf, uint32_t u32Size)
{
    uint8_t u8Cmd;
	  
		USBD_MemCopy((uint8_t *)&gCmd, pu8EpBuf, u32Size);
	
    /* Get command information */
    u8Cmd = gCmd.u8Cmd;

		/* Get Audio Volume */
		if(u8Cmd == HID_CMD_AudioVolume)
		{
				/* Get Volume */
				MainVolume = gCmd.u8AudioMain;
				LeftChannelVolume = gCmd.u8AudioLeft;
				RightChannelVolume = gCmd.u8AudioRight;
//				/* Feedback ACK */
//        gCmd.u8Cmd = HID_CMD_ACK;
//				/* Trigger HID IN */
//        USBD_MemCopy((uint8_t *)(USBD_BUF_BASE + USBD_GET_EP_BUF_ADDR(EP2)), (void *)&gCmd, EP2_MAX_PKT_SIZE);
//        USBD_SET_PAYLOAD_LEN(EP2, EP2_MAX_PKT_SIZE);
		}
    /* Get APROM FW version */
//    else if(u8Cmd == HID_CMD_GETFW)
//    {
//        /* Feedback FW version */
//        gCmd.u8Cmd = HID_CMD_ACK;
//			  gCmd.Version = FW_Version;
//				/* Trigger HID IN */
//        USBD_MemCopy((uint8_t *)(USBD_BUF_BASE + USBD_GET_EP_BUF_ADDR(EP2)), (void *)&gCmd, EP2_MAX_PKT_SIZE);
//        USBD_SET_PAYLOAD_LEN(EP2, EP2_MAX_PKT_SIZE);
//    }
//		else if(u8Cmd == HID_CMD_GETPARTNO)
//		{
//				/* Feedback Part NO. */
//        gCmd.u8Cmd = HID_CMD_ACK;
//				gCmd.Data[0] = (PDID >> 24) & 0xFF;
//				gCmd.Data[1] = (PDID >> 16) & 0xFF;
//				gCmd.Data[2] = (PDID >>  8) & 0xFF;
//				gCmd.Data[3] = (PDID >>  0) & 0xFF;
//				/* Trigger HID IN */
//        USBD_MemCopy((uint8_t *)(USBD_BUF_BASE + USBD_GET_EP_BUF_ADDR(EP2)), (void *)&gCmd, EP2_MAX_PKT_SIZE);
//        USBD_SET_PAYLOAD_LEN(EP2, EP2_MAX_PKT_SIZE);
//		}
//		else if(u8Cmd == HID_CMD_ISP)
//		{
//				/* Lower the protity of USB interrupt */
//				NVIC_SetPriority(USBD_IRQn, 1);
//			
//				/* Process the data phase of ISP command */
//				/* Close LED Effect */
//				PWM0->CMPDAT[0] = 0;		// JRGB1_R
//				PWM0->CMPDAT[1] = 0;		// JRGB1_G
//				PWM0->CMPDAT[2] = 0;		// JRGB1_B
//				PWM0->CMPDAT[3] = 0;		// Board_R
//				PWM0->CMPDAT[4] = 0;		// Board_G
//				PWM0->CMPDAT[5] = 0;		// Board_B
////				PWM1->CMPDAT[0] = 0;		// JRGB2_R
////				PWM1->CMPDAT[1] = 0;		// JRGB2_G
////				PWM1->CMPDAT[2] = 0;		// JRGB2_B
//			
//				/* LED_DATA2 goes first to avoid LED_DATA1 scatter-gather table be empty */
//				while(fPDMA_LED_DATA2_Done == 0);
//				Disable_LED_Data2();
//				while(fPDMA_LED_DATA2_Done == 0);
//			
//				while(fPDMA_LED_DATA1_Done == 0);
//				Disable_LED_Data1();
//				while(fPDMA_LED_DATA1_Done == 0);

//				/* Set the protity of USB interrupt */
//				NVIC_SetPriority(USBD_IRQn, 0);
//								
//				/* Boot from LDROM */
//				SYS_UnlockReg();
//				FMC_Open();
//				FMC_SetBootSource(1);		// Boot from LDROM
//				NVIC->ICPR[0] = 0xFFFFFFFF;		// Clear Pending Interrupt
//				/* Set VECMAP to LDROM for booting from LDROM */
//				FMC_SetVectorPageAddr(FMC_LDROM_BASE);
//				SYS_ResetCPU();
//		}
    else
    {
				printf("Unknown HID command!\n");
    }
}
